      SUBROUTINE HBLOCK(ID,IC,N,A,I1)
C
C
C  DEFINES, ADDS, SETS, GETS, OR CLEARS A USER BLOCK WITH IDENTITY ID
C
C
C  ID   (INTEGER)  IS THE IDENTIFIER
C  IC   (INTEGER)  CONTROL WORD SPECIFYING THE OPERATION:
C                  IC = 'ADD '  ADDS TO THE BLOCK IN FLOATING POINT
C                       'IADD'  ADD AS INTEGER TO BLOCK
C                  IC = 'SET '  SETS VALUES IN THE BLOCK
C                  IC = 'GET '  GETS VALUES FROM BLOCK
C                  IC = 'CLR '  CLEARS THE BLOCK TO ZEROS
C                       'CLE '     "    "    "    "   "
C                  IC = 'DEF '  DEFINES BLOCK (HCLR CANNOT CLEAR IT)
C                  IC = 'DEF0'  DEFINES BLOCK (HCLR CAN    CLEAR IT)
C                  IC = 'GETN'  GETS SIZE OF BLOCK AND TITLE
C  N    (INTEGER)  NUMBER OF ITEMS BEING DEFINED, ADDED, SET, ETC.
C                  (IF NOT SUPPLIED, THEN NUMBER IS TAKEN AS THAT
C                   USED IN THE DEF OPERATION FOR THIS ID).
C  A    REAL       IS THE ADDRESS OF THE VARIBALE OR ARRAY TO BE
C                  ADDED TO, SENT TO, OR RETRIEVED FROM THE BLOCK.
C                  (IF IC='IADD', THEN THE ADDITION IS DONE IN INTEGER
C                   MODE).
C  I1   (INTEGER)  FIRST ITEM IN BLOCK FOR THE ADD, SET, ETC.
C                  (IF NOT SUPPLIED, THE DEFAULT IS 1)
C
C
C  RETURN CODES:
C      IRETCD = 0  - ALL IS WELL
C               1  - ILLEGAL ID
C               2  - ILLEGAL IC ARG
C               3  - CANT DO OPERATION SINCE BLOCK NOT DEFINED
C               4  - INSUFFICIENT STORAGE FOR THE BLOCK
C               5  - BLOCK ALREADY EXISTS
C               6  - OUT-OF-BOUNDS ARGUMENT VALUES
C               7  - INSUFFICIENT NUMBER OF ARGUMENTS FOR THE OPTION
C               8  - ID IS NOT A BLOCK NODE
C
C
#include      "hcom.INC"
      INCLUDE 'dpmode.INC'
      INCLUDE 'hretcd.INC'
C
      REAL A(1)
      INTEGER IC
      CHARACTER*4 OPTS(9)
C
      CHARACTER*4 CTEMP
      EQUIVALENCE (ITEMP,RTEMP)
C
      INTEGER HPAK,ALL,HPNTR,A4ID,BLANK
C
      DATA OPTS/'ADD ','IADD','SET ','GET ','CLE ',
     >          'CLR ','DEF ','DEF0','GETN'/
      DATA HPAK/4HHPAK/, NWNODE/30/, BLANK/4H    /, ALL/4HALL /
C
      NARG=NARGS()
      IF(NARG.LT.0) NARG=5
      IF(HPAK.NE.HPNEW) CALL HINIT(2000)
C
      A4ID=ID
      IRETCD=1
      IF(ID.EQ.0.OR.ID.EQ.ALL) RETURN
      CALL HID2A4(ID,A4ID)
      WRITE(CTEMP,'(A4)') IC
      CALL DPUPPR(CTEMP)
C
C      FIND THE OPTION IN THE OPTION LIST
      DO 20 IOPT=1,9
   20 IF(CTEMP.EQ.OPTS(IOPT)) GO TO 50
      WRITE(ERUNIT,25) IC,A4ID
   25 FORMAT(/,' HBLOCK ERROR - ILLEGAL IC ARG  ',A4,'  FOR ID ',A4)
      IRETCD=2
      RETURN
C
   50 IF(HPNTR(ID).NE.0) GO TO 500
      IF(IOPT.GE.7.AND.IOPT.LE.8) GO TO 80
C
C       A NON-DEFINE OPERATION ON A NON-EXISTENT BLOCK
      WRITE(ERUNIT,55) IC,A4ID
   55 FORMAT(/,' HBLOCK ERROR - CANT DO ',A4,' SINCE BLOCK NOT DEFINED'
     >        ,' FOR ID=',A5)
      IRETCD=3
      RETURN
C
   80 IF(NARG.LT.3) GO TO 920
      NWTOT=NWNODE+N
C
C        CHECK AVAILABLE SPACE--QUIT IF FULL
      IOVER=NODEND+NWTOT-NHCOM
      IF(IOVER.LT.0) GO TO 400
      WRITE(ERUNIT,340) NHCOM,IOVER,A4ID
  340 FORMAT(/,' HCOM(',I5,') OVER BY',I5,' WDS FOR ID=',
     >      A4,' - HBLOCK')
      IRETCD=4
      RETURN
C
C
C
 
C       CONSTRUCT NEW NODE
C
  400 NNODES=NNODES+1
C
C       GET LOC OF CURRENT LAST NODE, AND MAKE IT LINK TO NEW ONE
      LNODE=M(NODEND+SHMOFS+1)+SHMOFS
      M(LNODE+3)=NODEND
      NODE=NODEND+SHMOFS
      IF(M(HSHCNT+SHMOFS+HASHID).GT.0) GO TO 410
C
C       FIRST ENTRY FOR THIS HASHID -- NODE POINTS TO ITSELF
      M(NODE+1)=NODE-SHMOFS
      GO TO 415
C
C       LINK THIS NODE INTO EXISTING HASH CHAIN (BREAK CHAIN TO DO IT)
  410 NODBRK=M(HSHLOC+SHMOFS+HASHID)+SHMOFS
      M(NODE+1)=M(NODBRK+1)
      M(NODBRK+1)=NODE-SHMOFS
C
  415 M(HSHLOC+SHMOFS+HASHID)=NODE-SHMOFS
      M(HSHCNT+SHMOFS+HASHID)=M(HSHCNT+SHMOFS+HASHID)+1
C
      M(NODE+2)=HASHID
      M(NODE+3)=NODE1
      M(NODE+4)=ID
      M(NODE+5)=0
      M(NODE+6)=A4ID
      M(NODE+7)=NWTOT
      M(NODE+8)=N
      M(NODE+9)=IOPT-7
      M(NODE+10)=0
C       COPY TITLE INTO 20 WORDS OF THE NODE
      ITEMP=BLANK
      DO 420 I=1,20
      IF(NARG.GE.4) RTEMP=A(I)
  420 M(NODE+10+I)=ITEMP
      NODEND=NODEND+NWTOT
C       NEXT NODE MUST BEGIN ON EVEN WORD BOUNDARY (I.E. NODEND EVEN)
      NODEND=NODEND+MOD(NODEND,2)
C
C       SAVE BACKWARD POINTER TO THIS NEW LAST NODE
      M(NODEND+SHMOFS+1)=NODE-SHMOFS
C
C       GO CLEAR THE BLOCK
      IOPT=6
C
C
C ------------------------------------------------------------------
C
C  THIS SECTION MODIFIES/RETRIEVES THE BLOCK
C
C ------------------------------------------------------------------
C
  500 IF(IOPT.LE.6.OR.IOPT.GE.9) GO TO 510
      WRITE(ERUNIT,505) A4ID
  505 FORMAT(/,' HBLOCK ERROR - ',A4,' ALREADY EXISTS')
      IRETCD=5
      RETURN
C
  510 J1=0
      IF(NARG.GE.5) J1=I1-1
      IF(M(NODE+5).NE.0) GO TO 940
      NN=M(NODE+8)
      IF(NARG.GE.3.AND.IOPT.NE.9) NN=N
      IF(J1.LT.-20 .OR. NN.GT.M(NODE+8)-J1) GO TO 900
      J=NODE+30+J1
      IRETCD=0
      GO TO (550,600,650,700,750,750,520,520,800),IOPT
  520 RETURN
C
C      ADD (REAL)
  550 IF(NARG.LT.4) GO TO 920
      DO 560 I=1,NN
  560 R(J+I)=R(J+I)+A(I)
      RETURN
C
C      IADD (INTEGER ADD)
  600 IF(NARG.LT.4) GO TO 920
      DO 610 I=1,NN
      RTEMP=A(I)
  610 M(J+I)=M(J+I)+ITEMP
      RETURN
C
C      SET
  650 IF(NARG.LT.4) GO TO 920
      DO 660 I=1,NN
  660 R(J+I)=A(I)
      RETURN
C
C      GET
  700 IF(NARG.LT.4) GO TO 920
      DO 710 I=1,NN
  710 A(I)=R(J+I)
      RETURN
C
C      CLR OR CLEAR
  750 DO 760 I=1,NN
  760 M(J+I)=0
      RETURN
C
C      GETN - STORE SIZE IN N AND TITLE IN A
  800 CONTINUE
      IF(NARG.LT.3) GO TO 920
      N=M(NODE+8)
      IF(NARG.LT.4) RETURN
      DO 810 I=1,20
  810 A(I)=R(NODE+10+I)
      RETURN
C
C      OUT OF BOUNDS - BAD ARGS
  900 IF(NARG.GE.5) WRITE(ERUNIT,910) A4ID,N,I1
      IF(NARG.LT.5) WRITE(ERUNIT,910) A4ID,N
  910 FORMAT(/,' HBLOCK ERROR - BAD OUT-OF-BOUNDS ARGS',
     >       ' ID/N/I1=',A5,2I7)
      IRETCD=6
      RETURN
C
C      INSUFFICIENT NUMBER OF ARGUMENTS
  920 WRITE(ERUNIT,930) A4ID
  930 FORMAT(/,' HBLOCK ERROR - INSUFFICIENT NUMBER OF ARGS FOR ID=',A5)
      IRETCD=7
      RETURN
C
C      CANT MODIFY NODE UNLESS IT IS A BLOCK NODE
  940 WRITE(ERUNIT,950) M(NODE+6)
  950 FORMAT(/,' HBLOCK ABORTED - ID=',A5,' IS NOT A BLOCK NODE')
      IRETCD=8
      RETURN
C
      END
