      SUBROUTINE DCURVE(UPLOT,MAXP,CORNER)
      REAL CORNER(2,4)
C
C   **MEMBER** DCURVE
C
C  DRAWS CURVE IN THE X,Y PLANE ALONG THE ZERO VALUES OF FUNCTION UPLOT.
C  UPLOT MUST BE OF THE FORM
C
C      REAL FUNCTION UPLOT(X,Y,N)
C  WHERE X,Y ARE THE COORDINATES AT WHICH UPLOT RETURNS THE FUNCTION
C  VALUE, AND N IS (OPTIONAL) THE CURVE NUMBER, BEGINING AT 1.
C
C  THIS CODE WAS ADAPTED FROM THE KIOWA PACKAGE.
C
C
      INCLUDE 'dpmode.INC'
C
      EXTERNAL UPLOT
      REAL SMIN(2),SMAX(2),GMIN(2),DGDS(2)
      EQUIVALENCE (XL,SMIN(1)),(YL,SMIN(2)),(XU,SMAX(1)),(YU,SMAX(2)),
     >  (GXMN,GMIN(1)),(GYMN,GMIN(2)),(DGDX,DGDS(1)),(DGDY,DGDS(2))
C
      REAL XDRAW(2),YDRAW(2)
C
      DIMENSION COSLST(19),SINLST(19)
C
      DATA HNONE/4HNONE/,MXPT/2000/
C
C        COSLST AND SINLST ARE COS(THETA) AND SIN(THETA) FOR
C        THETA=1.02,2,3,5,7,9,11,13,15,30,45,60,...,175 DEGREES
C       THE FIRST ANGLE MUST BE A LITTLE LARGE, TO AVOID A
C       PATHOLOGICAL CASE WHICH CAUSES OSCILLATION.
      DATA COSLST /
     X    0.999841542203  , 0.999390827019  , 0.998629534755
     X  , 0.996194698092  , 0.992546151641  , 0.987688340595
     X  , 0.981627183448  , 0.974370064785
     X  , 0.965925826289  , 0.866025403784  , 0.707106781187
     X  , 0.500000000000  , 0.258819045103  , 0.000000000000
     X  ,-0.258819045103  ,-0.500000000000  ,-0.707106781187
     X  ,-0.866025403784  ,-0.965925826289  /
      DATA SINLST /
     X    0.017801418053  , 0.034899496703  , 0.052335956243
     X  , 0.087155742748  , 0.121869343405  , 0.156434465040
     X  , 0.190808995377  , 0.224951054344
     X  , 0.258819045103  , 0.500000000000  , 0.707106781187
     X  , 0.866025403784  , 0.965925826289  , 1.000000000000
     X  , 0.965925826289  , 0.866025403784  , 0.707106781187
     X  , 0.500000000000  , 0.258819045103  /
C
C                            SET UP LOCAL VARIABLES
      IF(MAXP.LE.0) RETURN
      DO 5 I=1,2
      SMIN(I) = ASCALE(1,I)
      SMAX(I) = SMIN(I) + ASCALE(2,I)*ASCALE(3,I)
      DGDS(I) = (CORNER(I,3) - CORNER(I,1))/(SMAX(I) - SMIN(I))
    5 GMIN(I) = CORNER(I,1) - SMIN(I)*DGDS(I)
C
C                  PROTECT AGAINST UNREASONABLE NUMBER OF CURVES
      MAXPL=MIN0(25,MAX0(1,MAXP))
C
C                  DO THE CURVES, ONE BY ONE
      DO 1000 NPL=1,MAXPL
C                            RX AND RY ARE STEPS FOR A 300 BY 300 GRID
      RX = (XU-XL)*0.00331
      RY = (YU-YL)*0.00331
C                            DX AND DY ARE STEPS FOR A 30 BY 30 GRID
      DX = 10. * RX
      DY = 10. * RY
C                  THE GRID IS MADE SLIGHTLY INSIDE THE PLOT BOUNDS
C                  BECAUSE IF THE STARTING POINT IS EXACTLY ON THE
C                  EDGE, IT MAY BE SKIPPED WHEN COMING BACK TO IT IN
C                  DRAWING THE CURVE, AND THE DRAWER WONT SEE IT AS
C                  THE STOPPING POINT.
C
C                            STARTING POINT FOR COARSE SCAN
 10   YSTRT = YL + RY
      X = XL + RX
C                            GET FUNCTION VALUE AT THIS POINT.
      USV = UPLOT(X,YSTRT,NPL)
C                            CHECK FOR FLAG SAYING NO CURVE
      IF (USV.EQ.HNONE) GO TO 1000
C                            USE THE SIGN ONLY
      USV = SIGN(1.,USV)
C                  MAKE COARSE SCAN. COLUMNS FROM BOTTOM TO TOP
C                  AND THEN MOVE LEFT TO RIGHT
      DO 30 I=1,30
C                            START AT BOTTOM OF NEXT COLUMN
         Y = YSTRT
         X = X + DX
C                            CHANGE SINCE LAST COLUMN?
         IF (SIGN(1.,UPLOT(X,Y,NPL)).NE.USV) GO TO 40
C                            SCAN FROM BOTTOM UP
         DO 20 J=1,30
            Y = Y + DY
C                            DOES IT CHANGE SIGN?
            IF (SIGN(1.,UPLOT(X,Y,NPL)).NE.USV) GO TO 50
 20      CONTINUE
 30   CONTINUE
C                            NEVER FOUND A SIGN CHANGE
         GO TO 70
C                  FOUND A SIGN CHANGE BETW (X,Y) AND (X-DX,Y)
 40   DY = 0.
      GO TO 55
C                  FOUND A SIGN CHANGE BETW (X,Y) AND (X,Y-DY)
 50   DX = 0.
C                            NOW BETW (X,Y) AND (X-DX,Y-DY)
C                            LOOK BACKWARD ALONG THAT LINE IN STEPS
C                            0.1 AS BIG
 55   DX = -0.1*DX
      DY = -0.1*DY
      DO 60 J=1,10
         Y = Y + DY
         X = X + DX
         IF (SIGN(1.,UPLOT(X,Y,NPL)).EQ.USV) GO TO 100
 60   CONTINUE
C                            COULDNT FIND IT.
 70      WRITE(ERUNIT,80) NPL,IDA
 80      FORMAT ('0UNABLE TO FIND CURVE',I3,' FOR PLOT ID=',A6)
         GO TO 1000
C                            (XP,YP) IS BEST GUESS FOR STARTING POINT
 100  XP = X - 0.5*DX
      YP = Y - 0.5*DY
C                            SAVE STARTING POINT
      XI = XP
      YI = YP
C                            PICK ARBITRARY STARTING DIRECTION
      CP = 1.0
      SP = 0.0
C                            COUNT POINTS IN A ROW OUT OF BOUNDS
      IPOUT = 0
C                            COUNT TIMES THAT BOUNDS HAVE BEEN EXCEEDED
      ITOUT = 0
C                            COUNT VECTORS PLOTTED THIS CURVE
      IPTB = 1
C                            COUNT POINTS SINCE LEAVING START
      IPTT = 1
C                            USE ABS VALUES SO TEST FOR ENDPOINT WORKS
      RX = ABS(RX)
      RY = ABS(RY)
C                            COUNT POINTS IN THIS LINE SEGMENT
140   IPT = 1
C                            SET FIRST POINT.
      XPT = XP*DGDX + GXMN
      YPT = YP*DGDY + GYMN
      XDRAW(2)=XPT
      YDRAW(2)=YPT
C                  STEP AT SMALL ANGLES WRT CURRENT DIRECTION
 150  IFLG = 0
 160  DO 200 I=1,19
         CR = COSLST(I)
         SR = SINLST(I)
C                            GET NEW DIRECTION
         CN = CP*CR - SP*SR
         SN = SP*CR + CP*SR
C                            MAKE STEP
         XN = XP + RX*CN
         YN = YP + RY*SN
C                            VALUE AT NEW POINT
         UN = SIGN(1.,UPLOT(XN,YN,NPL))
C                            SAVE FIRST FOR REFERENCE VALUE
         IF (I.EQ.1) USV = UN
C                            DOES IT CHANGE SIGN?
         IF (UN.NE.USV) GO TO 250
C                  TRY THE SAME DEVIATION IN THE OTHER SIDE
         SR = -SR
C                            GET NEW DIRECTION
         CN = CP*CR - SP*SR
         SN = SP*CR + CP*SR
C                            MAKE STEP
         XN = XP + RX*CN
         YN = YP + RY*SN
C                            DOES IT CHANGE SIGN AT NEW POINT?
         IF (SIGN(1.,UPLOT(XN,YN,NPL)) .NE. USV) GO TO 250
C                            LOOP FOR NEXT LARGER DEVIATION
 200  CONTINUE
C                            DIDNT WORK.  A CUSP COULD CAUSE THIS.
         WRITE(ERUNIT,80) NPL,IDA
         GO TO 1000
C                            GOT THE CHANGE.  BACK UP HALF A STEP.
 250  SR = SIGN(SINLST(1),SR)
      CP = CN*COSLST(1) + SN*SR
      SP = SN*COSLST(1) - CN*SR
      IFLG = IFLG + 1
C                            IF THIS IS A LARGE DEVIATION, WITH LOW
C                            PRECISION, DONT STEP.  GO BACK AND
C                            DO IT BETTER FROM NEW DIRECTION.
      IF (CR.LT.0.966) GO TO 160
      XP = XP + RX*CP
      YP = YP + RY*SP
      IPTT = IPTT + 1
C                            CHECK FOR OUT OF BOUNDS
      IF ((XP-XU)*(XP-XL).GT.0.) GO TO 270
      IF ((YP-YU)*(YP-YL).GT.0.) GO TO 270
C                            IF NOT SAME DIRECTION AS LAST STEP, JUMP
      IF (I.NE.1) GO TO 260
      IF (IFLG.NE.1) GO TO 260
C                            STRAIGHT LINE, SKIP PLOTTING AND ADD TO IT
      GO TO 262
 260  IPT = IPT + 1
      IPTB = IPTB + 1
      XDRAW(1)=XDRAW(2)
      YDRAW(1)=YDRAW(2)
      XDRAW(2)=XPT
      YDRAW(2)=YPT
C..#IFDEF DGSYS
      CALL DGLNS(XDRAW,YDRAW,2)
CC..#ELIF GKSYS
C      CALL GPL(2,XDRAW,YDRAW)
CC..#ELSE
CWarning from FPPIFY. Replace or delete this line as needed.
C..#ENDIF
 262  XPT = XP*DGDX + GXMN
      YPT = YP*DGDY + GYMN
      IF (IPTT.LE.20) GO TO 150
C                            DONT GO ON FOREVER
      IF(IPTB.GT.MXPT) GO TO 500
C                            ARE WE CLOSE TO THE START?
      IF (ABS(XP-XI).GT.RX) GO TO 150
      IF (ABS(YP-YI).GT.RY) GO TO 150
C                            YES.  CONNECT TO IT.
      XPT = XI*DGDX + GXMN
      YPT = YI*DGDY + GYMN
C                            AND QUIT.
      GO TO 500
C                            OUT OF BOUNDS
  270 IF (IPT.GT.1) THEN
         XDRAW(1)=XDRAW(2)
         YDRAW(1)=YDRAW(2)
         XDRAW(2)=XPT
         YDRAW(2)=YPT
C..#IFDEF DGSYS
         CALL DGLNS(XDRAW,YDRAW,2)
CC..#ELIF GKSYS
C         CALL GPL(2,XDRAW,YDRAW)
CC..#ELSE
CWarning from FPPIFY. Replace or delete this line as needed.
C..#ENDIF
      ENDIF
      IPT = 0
      IPOUT = IPOUT + 1
      IF (IPOUT.LT.1000) GO TO 150
C                            AND FAR AWAY.
      ITOUT = ITOUT + 1
C                            FIRST TIME FOR THIS?
      IF (ITOUT.GT.1) GO TO 500
C                            YES.  GO BACK TO STARTING POINT
            XP = XI
            YP = YI
C                            WITH OPPOSITE DIRECTION
            CP = -1.
            SP = 0.
C                             AND START AGAIN.
            IPTT = 0
            GO TO 140
C
C                            FINISH LAST POINT AND QUIT
  500 XDRAW(1)=XDRAW(2)
      YDRAW(1)=YDRAW(2)
      XDRAW(2)=XPT
      YDRAW(2)=YPT
C..#IFDEF DGSYS
      CALL DGLNS(XDRAW,YDRAW,2)
CC..#ELIF GKSYS
C      CALL GPL(2,XDRAW,YDRAW)
CC..#ELSE
CWarning from FPPIFY. Replace or delete this line as needed.
C..#ENDIF
 1000 CONTINUE
C
      RETURN
      END
