      INTEGER FUNCTION DPLEXP(FN,I1,I2,J1,J2,NPLACE,KB,NDEC,KBASE,KS,NA)
C
C   **MEMBER** DPLEXP
C
C
C FIND EXPONENT TO REPRESENT THE VALUES OF THE FUNCTION FN(I,J)
C IN THE RANGE I=I1,I2 AND J=J1,J2 SO THAT EACH VALUE CAN BE REPRESENTED AS
C MANTISSA*10**EXPONENT.
C NPLACE IS THE MAXIMUM # OF PLACES ALLOWED IN THE MANTISSA.
C DPLEXP WILL CHOOSE EXPONENT SO THAT NPLACE SIGNIFICANT DIGITS
C ARE USED, BUT WILL SUPRESS INSIGNIFICANT LEADING OR TRAILING
C ZEROES.
C KB RETURNS THE NUMBER OF BYTES NEED TO HOLD THE LARGEST MANTISSA.
C   AND IF ARGUMENTS ARE ILLEGAL OR FN IS ALWAYS AERO, KS IS SET TO 0.
C NDEC RETURNS THE NUMBER OF DECIMAL PLACES (0 TO 7, 0 MEANS NO
C      DECIMAL POINT, 1 MEANS THE POINT BUT NO DECIMAL DIGITS).
C KBASE SPECIFIES THE BASE FOR THE MANTISSA (E.G. 10 FOR DECIMAL)
C       THE EXPONENT IS ALWAYS IN DECIMAL BASE HOWEVER.
C KS IS A SCALE FACTOR THAT ALLOWS GREATER RESOLUTION FOR THE
C CASE WHEN NPLACE IS 1 OR 2. (USED BY DPLP2 ONLY).
C NA SPECIFIES HOW MANY ARGUMENTS ARE TO BE USED
C
      INCLUDE 'dpmode.INC'
C
      EXTERNAL FN
      LOGICAL SLCFLG,TRUNC
      INTEGER ILEN(6)
      DATA ILEN/1,11,111,1111,11111,111111/
C
C
      SLCFLG=(SLFLAG.EQ.0.OR.J2.GT.J1)
C
C  MAX SINGLE PRECISION FLOATING PT NUMBER IS 6 DIGITS
      NPL=MIN0(6,NPLACE)
      DPLEXP=100
      NB=NPL
      IF(NA.GE.8) NDEC=0
      IBASE=10
      IF(NA.GE.9) IBASE=KBASE
      MS=1
      IF(I2.LT.I1 .OR. J2.LT.J1) GO TO 119
      FBASE=IBASE
C
C  GET MAX ABSOLUTE VALUE OF FN
C
      FMIN=0.
      FMAX=0.
      DO 5 J=J1,J2
      DO 5 K=I1,I2
      IF(.NOT.SLCFLG) F=DPSLCF(FN,K)
      IF(SLCFLG) F=FN(K,J)
      FMIN=AMIN1(FMIN,F)
    5 FMAX=AMAX1(FMAX,ABS(F))
      IF(FMAX.EQ.0.) GO TO 119
      IF(IBASE.EQ.10) GO TO 10
C
C  TREAT NON-DECIMAL CASE SEPARATELY
C
      NB=ALOG(FMAX)/ALOG(FBASE) + 1.0
      IF(NPL.LE.2.AND.NA.GE.10) MS=FMAX/FBASE**NPL + .99999
      DPLEXP=0
      GO TO 120
C
   10 TEMP=FBASE**NPL
      IEXP=IFIX(ALOG10(FMAX/TEMP)+100.001) - 100
C       FIX UP FOR TRUNCATION LOSS WHEN FMAX IS AN EXACT DECADE
      IF(FMAX/10.**(IEXP+1) .GE. TEMP) IEXP=IEXP+1
      IF(FMIN.GE.0.0 .OR. NPL.LE.1) GO TO 20
C       TEST IF HIGH ORDER PLACE SHOULD BE RESERVED FOR MINUS SIGN
      IF(-FMIN/10.**IEXP .GT. TEMP) IEXP=IEXP+1
   20 IEXP1=IEXP+1
      FHI=10.**IEXP1
C
C  LOOP OVER THE POINTS, LOOKING FOR LEAST SIGNIFICANT DIGITS
C
      TRUNC=.FALSE.
      DO 110 J=J1,J2
      DO 100 K=I1,I2
      IF(.NOT.SLCFLG) F=DPSLCF(FN,K)
      IF(SLCFLG) F=FN(K,J)
      IF(F.EQ.0.0) GO TO 100
      IEXP=IEXP1
      FAB=ABS(F)/FHI
C      TEST FOR PRECISION LOSS DUE TO TRUNCATION
      IF(AMOD(FAB,1.0).GT.0.1) TRUNC=.TRUE.
C     INTEGERIZE THE MANTISSA ALLOWING FOR ROUND OFF
      INTG=FAB+.5
      IF(INTG.EQ.0) GO TO 100
      IF(INTG.LT.1000) GO TO 40
C       FIX FOR ROUND OFF ERRORS IN 6'TH DIGIT
      ITEMP=MOD(INTG,100)
      IF(ITEMP.EQ.99) INTG=INTG+1
      IF(ITEMP.EQ.01) INTG=INTG-1
C     NOW LOOK FOR INSIGNIFICANT TRAILING ZEROES AND ADJUST
C     EXPONENT AND MANTISSA TO ELIMINATE THEM
   40 IF( (INTG/10)*10 .NE.INTG) GO TO 50
      IEXP=IEXP+1
      INTG=INTG/10
      GO TO 40
   50 DPLEXP=MIN0(DPLEXP,IEXP)
  100 CONTINUE
  110 CONTINUE
C
      NB=NPL-DPLEXP+IEXP1
      IF(NA.LE.9) GO TO 120
C
C  IF VALUES HAVE BEEN TRUNCATED, TRY USING A SCALE FACTOR
C  MS (KS) TO OBTAIN MORE PRECISION (WITH A SMALLER IEXP1).
C
      IF(.NOT.TRUNC .OR. NPL.GT.2) GO TO 120
      IF(NB.NE.NPL) GO TO 120
      MS=FMAX/10.**(NPL+DPLEXP-1) + .99999
      MS=MAX0(1,MS)
      IF(MS.GT.5) MS=1
      IF(MS.EQ.1) GO TO 120
      DPLEXP=DPLEXP-1
      IF(MS.GT.2) MS=5
      GO TO 120
C
  119 NB=0
  120 IF(DPLEXP.EQ.100) DPLEXP=0
      IF(NA.GE.7) KB=NB
      IF(NA.GE.10) KS=MS
C
C  USE ENGINEERING NOTATION IF BASE ARG IS NOT SUPPLIED
      IF(NA.NE.8.OR.NUMFMT.EQ.0) RETURN
      CALL DPDGIT(ILEN(NB),DPLEXP,IEXP,NDEC)
      KB=KB+MAX0(0,DPLEXP-IEXP)
      IF(NDEC.GT.0) KB=KB+1
      DPLEXP=IEXP
      RETURN
C
      END
