      INTEGER FUNCTION DPSCAL(F,NP,NJ,IAUT,ILG,IEDG,IZRO,NTIC,S,IS,LOW)
C
C   **MEMBER** DPSCAL
C
      EXTERNAL F
      REAL S(4)
      INTEGER IS(3),NTIC(2)
      REAL*8 BG,SM,TEMP,YMIN,DY
      REAL TIC(3)
C
C
C  FIND THE SCALE FACTORS FOR THE NP POINTS IN FUNCTION F, AND
C  SAVE THE RESULTS IN S.  ALSO FIND THE INTEGER-EXPONENT FORM
C  OF THESE SCALE FACTORS AND SAVE THESE IN IS. THE SCALING IS
C  CONDITIONAL ON THE VARIOUS OPTIONS SPECIFIED BY THE OTHER
C  ARGUMENTS.
C
C    F(I,J)   GIVES VALUE OF FUNCTION AT POINT I AND FUNCTION NUMBER J
C    NP       GIVES NUMBER OF POINTS
C    NJ       GIVES NUMBER OF DATA SETS
C    IAUT     CONTROLS WHETHER AUTO OR MANUAL SCALE FACTORS ARE TO BE
C             OBTAINED (SEE IS AND S DESCRIPTION).
C             0-DO MANUAL SCALING, S IS USED AS INPUT FOR IS.
C             1-AUTO SCALING, GENERATING TICS ON ENDS OF AXES, (DATA
C               MAY OCCUPY ON PART OF THE AXES' RANGE).
C             2-AUTO SCALING, SUCH THAT DATA FILLS THE FULL RANGE
C               OF THE AXES (TICS NOT ON ENDS OF AXES IN GENERAL).
C
C    ILG      0-DO LINEAR SCALING, 1-DO LOG SCALING
C    IEDG     0-USE ALL N POINTS AND ALL NF FUNCTIONS
C             1-OMIT POINT 1 AND N BUT USE ALL FUNCTIONS
C             2-OMIT FUNCTION 1 AND NF BUT USE ALL POINTS
C             3-OMIT POINTS 1 AND N AND OMIT FUNCTION 1 AND NF
C    IZRO     0-NO ZERO POINT RESTRICTION,
C             1-ADD AN ADDITIONAL ZERO VALUED POINT TO FORCE A
C               ZERO WITHIN THE RANGE OF THE SCALE FACTORS.
C    NTIC     SUPPLIES DEFAULT TICS AND SUBS TO BE USED WHEN IAUT GT 0
C                NTIC(1) IS THE NUMBER OF MAJOR TICS, IF ZERO THEN
C                        IT IS GENERATED INTERNALLY BY DPSCAL.
C                NTIC(2) IS THE NUMBER OF SUB-TICS, AND IF ZERO
C                        THEN DPSCAL SUPPLIES A VALUE AUTOMATICALLY.
C    S        SUPPLIES THE SCALE FACTORS IF IAUT=0, OR GETS FILLED
C             WITH NEW SCALE FACTORS IF IAUT>0, WHERE;
C                S(1) = SMIN, (VALUE TO BE ASSOCIATED WITH ORIGIN OF AXIS).
C                S(2) = DS    (DIFFERENCE IN VALUES BETWEEN ADJACENT TICS)
C                S(3) = TICS  (NUMBER OF TIC MARKS, MAY HAVE FRACTION)
C                S(4) = SUBS  (NUMBER OF SUB TIC INTERVALS).
C
C    IS       GETS FILLED WITH INTEGER-EXPONENT SCALE FACTORS;
C                IS(1) = ISMIN
C                IS(2) = IDS
C                IS(3) = IPWR
C              SUCH THAT SMIN GE ISMIN*10**IPWR, AND VALUE OF
C                        I'TH TIC IS (ISMIN+I*IDS)*10**IPWR
C
C    LOW       1-USE THE N+1 VALUE ALSO (HISTOGRAM X SCALING)
C              2-LEAVE TOP 10% FREE FOR TEXT (Y SCALING)
C              ANYTHING ELSE DOES NEITHER OF ABOVE.
C
      INCLUDE 'dpmode.INC'
      INCLUDE 'dcomfp.INC'
C
      LOGICAL SLCFLG
C
      DATA TIC/1.0,2.0,5.0/
C
C
      DPSCAL=0
      IAUTOT=IAUT
      IF(IAUT.EQ.0)GO TO 200
5     K=1
      K2=NP
      IF(K2.GT.0) GO TO 8
C       NO POINTS  - SET SCALES TO DEFAULT VALUES
      IF(ILG.NE.0) GO TO 85
      SM=0.D0
      BG=1.0D0
      GO TO 37
C
    8 IF(LOW.EQ.1) K2=K2+1
      IF(MOD(IEDG,2).EQ.0.OR.K2.LE.2) GO TO 10
      K=2
      K2=K2-1
   10 N=1
      NF=NJ
      IF(NF.LE.0) NF=1
      SLCFLG=(SLFLAG.EQ.0.OR.NF.GT.1)
      IF(IEDG.LT.2.OR.NF.LE.2) GO TO 15
      N=2
      NF=NF-1
   15 IF(ILG.EQ.1) GO TO 100
      SM=FPMAX
      BG=-FPMAX
      DO 30 J=1,NF
      DO 20 I=K,K2
      IF(SLCFLG) TEMP=F(I,J)
      IF(.NOT.SLCFLG) TEMP=DPSLCF(F,I)
      SM=DMIN1(TEMP,SM)
 20   BG=DMAX1(TEMP,BG)
 30   CONTINUE
      IF(IZRO.EQ.0) GO TO 35
      SM=DMIN1(SM,0.D0)
      BG=DMAX1(BG,0.D0)
C      RESTRICT RANGE TO 6 DIGITS OR LESS
   35 IF(BG-SM .LT. DMAX1(DABS(BG),DABS(SM))*1.D-6) BG=SM
C      EXTEND Y HIGH, IF LOW IS 2
      IF(LOW.EQ.2) BG=BG+0.1D0*(BG-SM)
      IF(BG.NE.SM) GO TO 37
      DPSCAL=1
      BG=SM+0.1*DABS(SM)
      IF(SM.EQ.0.D0) BG=1.D0
   37 IF(IAUTOT.GE.2) GO TO 55
      IT1=MAX0(1,NTIC(1))
      IT2=IT1
      IF(NTIC(1).GT.0) GO TO 40
      IT1=3
      IT2=8
   40 DY=FPMAX
      RBEST=0.0
C
      TEMP=0.0
      DO 50 ITIC=IT1,IT2
      IF(BG.EQ.SM)GO TO 45
      TEMP=DLOG10((BG-SM)/ITIC)
C
45    DO 60 I=1,3
      IPWR=1+DINT(TEMP-ALOG10(TIC(I))+100.)-100
      PWR=10.**IPWR
      YMIN=TIC(I)*PWR
      ITRUNC=DINT(SM/YMIN)
      IF (SM.LT.ITRUNC*YMIN) ITRUNC=ITRUNC-1
      IF (ITIC*YMIN.LT.BG-YMIN*ITRUNC) IPWR=IPWR+1
      PWR=10.**IPWR
      YMIN=TIC(I)*PWR
      TBEST=(BG-SM)/(YMIN*ITIC)
      IF (YMIN.GE.DY.OR.TBEST-RBEST.LT.0.00001) GO TO 60
      DY=YMIN
      IDY=TIC(I)
      RBEST=TBEST
      NINT=ITIC
      ISAV=IPWR
C
   60 CONTINUE
   50 CONTINUE
C
      ITRUNC=SM/DY
      IF (SM.LT.ITRUNC*DY) ITRUNC=ITRUNC-1
      YMIN=DY*ITRUNC
      IYMIN=YMIN/(10.**ISAV)+DSIGN(.001D0,YMIN)
      IPWR=ISAV
      IS(1)=IYMIN
      IS(2)=IDY
      IS(3)=IPWR
      S(1)=YMIN
      S(2)=DY
      S(3)=NINT
  53  S(4)=NTIC(2)
      IF(NTIC(2).GT.0) GO TO 400
      IF(S(3).GT.5.0) GO TO 400
      S(4)=2.0
      IF(S(3).GT.3.) GO TO 400
      S(4)=5.0
      IF(MOD(IS(2),2).EQ.0) S(4)=4.0
      GO TO 400
C
C ALTERNATE FORM OF SCALING - FULL FRAME DATA
   55 SBINS=NTIC(1)
      IF(SBINS.LE.0.) SBINS=8.0
      SMIN=SM
      SMAX=BG
      WBIN=0.
      IF(K2.GT.1) WBIN=(SMAX-SMIN)/(K2-1)
      IF(LOW.EQ.0) THEN
         SMIN=SMIN-WBIN/2.
         SMAX=SMAX+WBIN/2.
      ENDIF
      CALL DKSCAL(SMIN,SMAX,SBINS,0,IS)
      PWR=10.**IS(3)
      S(1)=SMIN
      S(2)=IS(2)*PWR
      S(3)=(SMAX-SMIN)/S(2)
      GO TO 53
C
C LOG SCALE
C
  100 XMAX=0.
      XMIN=FPMAX
      N0=0
C
      DO 75 J=N,NF
      DO 70 I=K,K2
      IF(SLCFLG) X=F(I,J)
      IF(.NOT.SLCFLG) X=DPSLCF(F,I)
      IF (X.GT.0.) GO TO 65
      N0=N0+1
      GO TO 70
   65 XMAX=AMAX1(X,XMAX)
      XMIN=AMIN1(XMIN,X)
   70 CONTINUE
   75 CONTINUE
      IF(N0.GT.0) XMIN=XMIN/4.
C
   80 IF (XMAX.NE.0.) GO TO 90
      DPSCAL=2
   85 XMAX=10.0
      XMIN=1.0
   90 YMIN=ALOG10(XMIN)
      YMAX=ALOG10(XMAX)
      T1=0.0
      IF (YMIN.LT.0.D0) T1=.999
      IS(1)=YMIN-T1
      IS(2)=1
      IS(3)=0
      IF(IAUTOT.GE.2) GO TO 95
      T1=0.0
      IF (YMAX.GT.0.) T1=.999
      MAX=YMAX+T1
      S(3)=MAX-IS(1)
      S(1)=IS(1)
      S(2)=1.0
      S(4)=5.
      IF(S(3).LE.2.) S(4)=9.
      RETURN
C
C LOG SCALING WITH FULL FRAME DATA
   95 S(1)=YMIN
      S(2)=1.0
      S(3)=YMAX-YMIN
      S(4)=5.
      IF(S(3).LE.2.) S(4)=9.
      RETURN
C
C       MANUAL SCALES
C
200   IF(S(2).LE.0.) GO TO 300
      IF(ILG.NE.0) GO TO 250
C
C       MANUAL LINEAR SCALES
      IS(3)=INT(ALOG10(S(2)) +100.) - 100
      I=0
  210 TMP=10.**IS(3)
      IF(I.GE.5) GO TO 220
      ITMP=S(2)/TMP + .5
      IF(ABS((ITMP*TMP-S(2))/S(2)).LE.0.001) GO TO 220
      IS(3)=IS(3)-1
      I=I+1
      GO TO 210
  220 IS(1)=S(1)/S(2)
      IF(IS(1)-S(1)/S(2).GT.0.0001) IS(1)=IS(1)-1
      T1=IS(1)*S(2)/TMP
      IS(1)=IS(1)*S(2)/TMP
      IF(S(1).LT.0.0.AND.ABS(IS(1)-T1).GT.0.0001) IS(1)=IS(1)-1
      IS(2)=S(2)/TMP + 0.5
      GO TO 400
C
C       MANUAL LOG SCALES
  250 IF(S(2).NE.1.) GO TO 300
      IS(1)=S(1)
      IF(S(1).LT.0.0 .AND. ABS(S(1)-IS(1)).GT.0.01) IS(1)=IS(1)-1
      IS(2)=1
      IS(3)=0
      RETURN
C
C       MANUAL SCALES ARE IMPOSSIBLE - DO AUTO SCALING INSTEAD
300   IAUTOT=1
      GO TO 5
C
C  ADJUST EXPONENT TO BE A MULTIPLE OF 3 (ENGINEERING NOTATION)
C      (IF LABEL FORMAT IS SO REQUESTED)
  400 IF(NUMFMT.EQ.0) RETURN
      K=S(3)+.0001
      IMX=MAX0(IABS(IS(1)),IABS(IS(1)+K*IS(2)))
      CALL DPDGIT(IMX,IS(3),IPWR,NDECI)
      IF(IPWR.GE.IS(3)) GO TO 410
      K=10**(IS(3)-IPWR)
      IS(1)=IS(1)*K
      IS(2)=IS(2)*K
  410 IS(3)=IPWR+1000*NDECI
      RETURN
C
      END
